
require "kaminari/models/array_extension"
require "kaminari/models/page_scope_methods"

module ActiveResource::Hel

  class Collection < ActiveResource::Collection
    include Kaminari::ConfigurationMethods::ClassMethods
    include Kaminari::PageScopeMethods

    attr_internal_accessor :limit_value, :offset_value

    attr_reader :parent, :attribute
    delegate :empty?, :to => :to_a

    def initialize(original_collection = [], options = {})
      @_original_collection, @_limit_value, @_offset_value, @_total_count = original_collection, (options[:limit] || default_per_page).to_i, options[:offset].to_i, options[:total_count]

      @model = options.fetch(:model, nil)
      @parent = options.fetch(:parent, nil)
      @attribute = options.fetch(:attribute, nil)
      @resource_class = original_collection.try(:resource_class)

      if options[:limit] && options[:offset]
        class << self
          include Kaminari::PageScopeMethods
        end
      end

      if options[:total_count]
        super original_collection.to_a
      else
        super(original_collection[@_offset_value, @_limit_value].to_a || [])
      end

    end

    def model
      @model ||= resource_class || to_a.first.try { |m| m.class }
    end

    # items at the specified "page"
    class_eval <<-RUBY, __FILE__, __LINE__ + 1
      def #{Kaminari.config.page_method_name}(num = 1)
        offset(limit_value * ([num.to_i, 1].max - 1))
      end
    RUBY

    def each_with_destroyed
      to_a.each
    end

    def entry_name
      (model.model_name.try(:human) || 'entry').downcase
    end

    def map
      self.class.new super, :model => model, :attribute => attribute, :parent => parent,
                     :limit => @_limit_value, :offset => @_offset_value, :total_count => @_total_count
    end

    # returns another chunk of the original collection
    def limit(num)
      self.class.new @_original_collection, :limit => num, :offset => @_offset_value, :total_count => @_total_count, :model => model
    end

    # total item numbers of the original collection
    def total_count
      @_total_count || @_original_collection.count
    end

    # returns another chunk of the original collection
    def offset(num)
      self.class.new @_original_collection, :limit => @_limit_value, :offset => num, :total_count => @_total_count, :model => model
    end

    def self.paginate_collection(array_or_collection, options = {})
      new array_or_collection, options
    end

#    private

    def mark_dirty
      unless parent.blank? || attribute.blank?
        parent.set_attribute(attribute, self)
      end
    end

  end
end