require 'active_support/core_ext/module/delegation'

module ActiveResource::Hel
  module Scopes
    extend ActiveSupport::Concern

    included do
      class_attribute :scopes
      self.scopes ||= {}

      class << self

        prepend PrependClassMethods
      end
    end

    module PrependClassMethods
      def inherited(subclass)
        super(subclass)
        subclass.scopes = self.scopes.dup
      end
    end

    module ClassMethods
      def scoped
        ActiveResource::Hel::Scope.new(self)
      end

      delegate :where, :to => :scoped
      delegate :set_filter, :to => :scoped


      def scopes
        self.scopes ||= {}
      end

      def scope(name, *args, &block)
        scopes[name = name.to_sym] = (args.first || block || lambda { |*args| where name => args })
        self.class.delegate name, :to => :scoped
      end
    end
  end

  class Scope
    include Enumerable

    attr_reader :resource_class
    attr_accessor :filters

    def initialize(resource_class)
      @resource_class = resource_class
    end

    def params
      @params ||= {}
    end

    def custom_options
      @custom_options ||= {}
    end

    def to_a
      all
    end

    delegate :each, :to => :to_a

    def first
      limit(1).all.first
    end

    def all(*args)
      args = (args.last if args.last.is_a?(Hash)) || {}
      params.merge!(:filter => filters.to_json) if filters
      resource_class.find :all, custom_options.merge(:params => params).merge(args)
    end

    def my
      params.merge!(:filter => filters.to_json) if filters
      dup.tap { |scope| scope.custom_options.merge!(:from => :my) }
    end

    def where(params = {})
      dup.tap { |scope| scope.params.update params }
    end

    def set_filter(filter)
      dup.tap do |scope| 
        if scope.filters.blank?
          scope.filters = filter
        else
          scope.filters = {:a => scope.filters, :o => 'AND', :b => filter }
        end
      end
    end

    delegate :scopes, :to => :resource_class

    def method_missing(name, *args)
      return super unless scopes.key? name
      scope = scopes[name]
      scope = scope.call(*args) if scope.is_a? Proc
      if scope.respond_to?(:filters) && scope.filters
        self.filters = set_filter(scope.filters).filters
      end

      where scope.params
    end
  end
end