module ActiveResource::Hel
  module Validations
    extend ActiveSupport::Concern

    module ClassMethods
      def remote_validation
        if defined?(@remote_validation)
          @remote_validation
        elsif superclass != ActiveResource::Base && (superclass.remote_validation != nil)
          superclass.remote_validation
        else
          @remote_validation = false
        end
      end

      def remote_validation=(value)
        @remote_validation = value
      end
    end

    def save_with_validation(options={})
      perform_validation = options[:validate] != false

      # clear the remote validations so they don't interfere with the local
      # ones. Otherwise we get an endless loop and can never change the
      # fields so as to make the resource valid.
      @remote_errors = nil
      if perform_validation && valid?(:skip_remote_validation => true) || !perform_validation
        save_without_validation
        true
      else
        false
      end
    rescue ResourceInvalid => error
      # cache the remote errors because every call to <tt>valid?</tt> clears
      # all errors. We must keep a copy to add these back after local
      # validations.
      @remote_errors = error
      load_remote_errors(@remote_errors, true)
      false
    end

    def valid?(options = {})
      opt = options.merge(:only_validate => true)
      @remote_errors = nil

      # FIXME - callback before_validate inkoved after remote validation
      if self.class.remote_validation && !options[:skip_remote_validation]
        begin
          new? ? create(opt) : update(opt)
        rescue ActiveResource::ResourceInvalid => error
          # cache the remote errors because every call to <tt>valid?</tt> clears
          # all errors. We must keep a copy to add these back after local
          # validations.
          @remote_errors = error
          load_remote_errors(@remote_errors, true)
        end
      end
      super()
    end

  end
end